Crate namada_node

Crate namada_node 

Library code for a Namada node.

Re-exports

pub use namada_apps_lib::tendermint_config;
pub use namada_apps_lib::tendermint_rpc;

Modules

ethereum_oracle
protocol
The ledger’s protocol
shell
The ledger shell connects the ABCI++ interface with the Namada ledger app.
shims
storage
The storage module handles both the current state in-memory and the stored state in DB.
tendermint
Re-export of tendermint v0.37
tendermint_node
tendermint_proto
Re-export of tendermint-proto v0.37
tower_abci
utils
Node utils commands handlers

Functions

dry_run_tx
Dry run a transaction
dump_db
Dump Namada ledger node’s DB from a block into a file
migrating_state
Determine if the ledger is migrating state.
query_db
reset
Resets the tendermint_node state and removes database files
rollback
Roll Namada state back to the previous height
run
Run the ledger with an async runtime
test_genesis_files
This function runs Shell::init_chain on the provided genesis files. This is to check that all the transactions included therein run successfully on chain initialization.