pub enum SnapshotRequest {
ListSnapshots,
OfferSnapshot(OfferSnapshot),
LoadSnapshotChunk(LoadSnapshotChunk),
ApplySnapshotChunk(ApplySnapshotChunk),
}Expand description
The snapshot category of ABCI requests.
Variants
ListSnapshots
Asks the application for a list of snapshots.
OfferSnapshot(OfferSnapshot)
Offers a list of snapshots to the application.
OfferSnapshot is called when bootstrapping a node using state sync. The
application may accept or reject snapshots as appropriate. Upon accepting,
Tendermint will retrieve and apply snapshot chunks via
ApplySnapshotChunk. The application may also choose to reject a snapshot
in the chunk response, in which case it should be prepared to accept further
OfferSnapshot calls.
Only app_hash can be trusted, as it has been verified by the light client.
Any other data can be spoofed by adversaries, so applications should employ
additional verification schemes to avoid denial-of-service attacks. The
verified app_hash is automatically checked against the restored application
at the end of snapshot restoration.
See also the Snapshot data type and the ABCI state sync documentation.
LoadSnapshotChunk(LoadSnapshotChunk)
Used during state sync to retrieve snapshot chunks from peers.
ApplySnapshotChunk(ApplySnapshotChunk)
Applies a snapshot chunk.
The application can choose to refetch chunks and/or ban P2P peers as appropriate. Tendermint will not do this unless instructed by the application.
The application may want to verify each chunk, e.g., by attaching chunk
hashes in Snapshot::metadata and/or incrementally verifying contents
against app_hash.
When all chunks have been accepted, Tendermint will make an ABCI Info
request to verify that last_block_app_hash and last_block_height match
the expected values, and record the app_version in the node state. It then
switches to fast sync or consensus and joins the network.
If Tendermint is unable to retrieve the next chunk after some time (e.g.,
because no suitable peers are available), it will reject the snapshot and try
a different one via OfferSnapshot. The application should be prepared to
reset and accept it or abort as appropriate.