Module public_key

Module public_key 

Public keys used in Tendermint networks

Structs

Ed25519
PubKeyRequest
PubKeyRequest requests the consensus public key from the remote signer.
PubKeyResponse
PubKeyResponse

Enums

Algorithm
Public key algorithms
PublicKey
Public keys allowed in Tendermint protocols
TendermintKey
Public key roles used in Tendermint networks

Functions

deserialize_public_key
Custom deserialization for public keys to handle multiple potential JSON formats from Tendermint.

Type Aliases

Secp256k1
ECDSA/secp256k1 verification key (i.e. public key)