pub enum Response {
Show 17 variants
Exception(Exception),
Echo(Echo),
Flush,
Info(Info),
InitChain(InitChain),
Query(Query),
CheckTx(CheckTx),
Commit(Commit),
ListSnapshots(ListSnapshots),
OfferSnapshot(OfferSnapshot),
LoadSnapshotChunk(LoadSnapshotChunk),
ApplySnapshotChunk(ApplySnapshotChunk),
PrepareProposal(PrepareProposal),
ProcessProposal(ProcessProposal),
ExtendVote(ExtendVote),
VerifyVoteExtension(VerifyVoteExtension),
FinalizeBlock(FinalizeBlock),
}Expand description
All possible ABCI responses for this protocol version.
Variants
Exception(Exception)
Returns an exception (undocumented, nondeterministic).
Echo(Echo)
Echoes a string to test an ABCI implementation.
Flush
Indicates that all pending requests have been completed with their responses flushed.
Info(Info)
Returns information about the application state.
InitChain(InitChain)
Returned on genesis after initializing chain state.
Query(Query)
Returns data queried from the application.
CheckTx(CheckTx)
Returns the result of checking a transaction for mempool inclusion.
Commit(Commit)
Returns the result of persisting the application state.
ListSnapshots(ListSnapshots)
Returns a list of local state snapshots.
OfferSnapshot(OfferSnapshot)
Returns the application’s response to a snapshot offer.
See also the OfferSnapshot data type and the ABCI state sync documentation.
LoadSnapshotChunk(LoadSnapshotChunk)
Returns a snapshot chunk from the application.
ApplySnapshotChunk(ApplySnapshotChunk)
Returns the result of applying a snapshot chunk and associated data.
The application can choose to refetch chunks and/or ban P2P peers as appropriate. Tendermint will not do this unless instructed by the application.
PrepareProposal(PrepareProposal)
ProcessProposal(ProcessProposal)
ExtendVote(ExtendVote)
VerifyVoteExtension(VerifyVoteExtension)
FinalizeBlock(FinalizeBlock)