Public keys used in Tendermint networks
Structs
- Ed25519
- PubKey
Request - PubKeyRequest requests the consensus public key from the remote signer.
- PubKey
Response - PubKeyResponse
Enums
- Algorithm
- Public key algorithms
- Public
Key - Public keys allowed in Tendermint protocols
- Tendermint
Key - Public key roles used in Tendermint networks
Functions
- deserialize_
public_ key - Custom deserialization for public keys to handle multiple potential JSON formats from Tendermint.
Type Aliases
- Secp256k1
- ECDSA/secp256k1 verification key (i.e. public key)